# -*- coding: utf-8 -*-
"""
imio.annex
----------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from imio.annex.interfaces import IAnnexFileChangedEvent
from imio.annex.interfaces import IConversionReallyFinishedEvent
from imio.annex.interfaces import IConversionStartedEvent
from zope.component.interfaces import ObjectEvent
from zope.interface import implements


class AnnexFileChangedEvent(ObjectEvent):
    implements(IAnnexFileChangedEvent)

    def __init__(self, object, file, called_by=None):
        """p_called_by can be used by notifier to specify where it was notified
           and so handlers may check this value and behave accordingly."""
        super(AnnexFileChangedEvent, self).__init__(object)
        self.file = file
        self.called_by = called_by


class ConversionStartedEvent(ObjectEvent):
    implements(IConversionStartedEvent)


class ConversionReallyFinishedEvent(ObjectEvent):
    implements(IConversionReallyFinishedEvent)



# -*- coding: utf-8 -*-
"""
imio.annex
----------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from collective.documentviewer.settings import GlobalSettings
from collective.documentviewer.utils import allowedDocumentType
from collective.iconifiedcategory.utils import get_category_object
from collective.iconifiedcategory.utils import update_categorized_elements
from imio.annex.events import AnnexFileChangedEvent
from plone import api
from zope.event import notify


def annex_content_created(obj, event):
    notify(AnnexFileChangedEvent(obj, obj.file))


def annex_content_updated(obj, event):
    if obj.file._blob._p_blob_uncommitted is not None:
        notify(AnnexFileChangedEvent(obj, obj.file))


def annex_file_changed(event):
    obj = event.object
    settings = GlobalSettings(api.portal.get())
    if not allowedDocumentType(obj, settings.auto_layout_file_types):
        return
    # do something?


def annex_conversion_started(obj, event):
    container = obj.aq_parent
    if obj.UID() not in getattr(container, 'categorized_elements', {}):
        return
    category = get_category_object(obj, obj.content_category)
    update_categorized_elements(container, obj, category)


def annex_conversion_really_finished(obj, event):
    container = obj.aq_parent
    if obj.UID() not in getattr(container, 'categorized_elements', {}):
        return
    category = get_category_object(obj, obj.content_category)
    update_categorized_elements(container, obj, category)



# -*- coding: utf-8 -*-
"""
imio.annex
----------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from plone import api


def is_iconified_categorized(portal_type):
    """
    Verify if the given portal_type have the IIconifiedCategorization behavior
    """
    portal_types = api.portal.get_tool('portal_types')
    pt = portal_types.get(portal_type)
    if not pt:
        return False
    behavior = ('collective.iconifiedcategory.behaviors.'
                'iconifiedcategorization.IIconifiedCategorization')
    return behavior in pt.behaviors



# -*- coding: utf-8 -*-
"""
imio.annex
----------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from collective.iconifiedcategory import _ as ICMF
from collective.iconifiedcategory.widget.widget import CategoryTitleFieldWidget
from collective.z3cform.select2.widget.widget import SingleSelect2FieldWidget
from imio.annex.quickupload import utils
from plone.z3cform.layout import FormWrapper
from z3c.form import field
from z3c.form.form import Form
from z3c.form.interfaces import HIDDEN_MODE
from zope import schema
from zope.interface import Interface


class IQuickUpload(Interface):

    title = schema.TextLine(
        title=u'Title',
        required=True,
    )

    content_category = schema.Choice(
        title=ICMF(u'Category'),
        source='collective.iconifiedcategory.categories',
        required=True,
    )

    default_titles = schema.Choice(
        title=ICMF(u'Default title'),
        vocabulary='collective.iconifiedcategory.category_titles',
        required=False,
    )

    description = schema.Text(
        title=u'Description',
        required=False,
    )


class QuickUploadForm(Form):
    fields = field.Fields(IQuickUpload)
    fields['content_category'].widgetFactory = SingleSelect2FieldWidget
    fields['default_titles'].widgetFactory = CategoryTitleFieldWidget
    fields['default_titles'].mode = HIDDEN_MODE
    ignoreContext = True

    @property
    def typeupload(self):
        return self.request.get('typeupload')

    @property
    def is_iconified_categorized(self):
        return utils.is_iconified_categorized(self.typeupload)

    def update(self):
        super(QuickUploadForm, self).update()
        if self.is_iconified_categorized is False:
            del self.widgets['content_category']
            del self.widgets['default_titles']


class QuickUploadFormView(FormWrapper):
    form = QuickUploadForm

    def update(self):
        super(QuickUploadFormView, self).update()



# -*- coding: utf-8 -*-

from collective.documentviewer.settings import GlobalSettings
from collective.iconifiedcategory.utils import get_categorized_elements
from plone import api
from zope.annotation import IAnnotations


def get_annexes_to_print(container, portal_type=None, sort_on=None, filters={"to_print": True}, caching=True):
    ''' '''
    res = None
    if caching:
        key = "utils-get_annexes_to_print-%s-%s-%s" % (
            container.UID(),
            portal_type,
            str(filters))
        cache = IAnnotations(container.REQUEST)
        res = cache.get(key, None)

    if res is None:
        res = []
        portal = api.portal.get()
        global_settings = GlobalSettings(portal)
        annexes = get_categorized_elements(container,
                                           result_type='dict',
                                           sort_on=sort_on,
                                           portal_type=portal_type,
                                           filters=filters)
        i = 1
        for annex_infos in annexes:
            # must have been converted successfully
            if not annex_infos['preview_status'] == 'converted':
                continue

            # every annex seems right, manage this annex build path to images
            # use unrestrictedTraverse to avoid failing if a folder is private
            # like it is the case for 'Members'
            annexObj = portal.unrestrictedTraverse(annex_infos['relative_url'])
            annex_annotations = IAnnotations(annexObj)
            data = {}
            data['title'] = annexObj.Title()
            annexUID = annexObj.UID()
            data['UID'] = annexObj.UID()
            data['number'] = i
            data['images'] = []
            data['number_of_images'] = annex_annotations['collective.documentviewer']['num_pages']
            image_format = annex_annotations['collective.documentviewer']['pdf_image_format']
            # we need to traverse to something like :
            # @@dvpdffiles/c/7/c7e2e8b5597c4dc28cf2dee9447dcf9a/large/dump_1.png
            dvpdffiles = portal.unrestrictedTraverse('@@dvpdffiles')
            filetraverser = dvpdffiles.publishTraverse(container.REQUEST, annexUID[0])
            filetraverser = dvpdffiles.publishTraverse(container.REQUEST, annexUID[1])
            filetraverser = dvpdffiles.publishTraverse(container.REQUEST, annexUID)
            large = filetraverser.publishTraverse(container.REQUEST, 'large')
            for image_number in range(data['number_of_images']):
                realImageNumber = image_number + 1
                large_image_dump = large.publishTraverse(
                    container.REQUEST, 'dump_%d.%s' % (realImageNumber, image_format))
                # depending on the fact that we are using 'Blob' or 'File' as storage_type,
                # the 'large' object is different.  Either a Blob ('Blob') or a DirectoryResource ('File')
                if global_settings.storage_type == 'Blob':
                    blob = large_image_dump.settings.blob_files[large_image_dump.filepath]
                    # if we do not check 'readers', the blob._p_blob_committed is sometimes None...
                    blob.readers
                    path = blob._p_blob_committed
                else:
                    path = large_image_dump.context.path
                data['images'].append({'number': realImageNumber,
                                       'path': path,
                                       })
            res.append(data)
            i = i + 1
        if caching:
            cache[key] = res
    return res



# -*- coding: utf-8 -*-
"""
imio.annex
----------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from AccessControl import Unauthorized
from Acquisition import aq_inner
from collective.iconifiedcategory.utils import validateFileIsPDF
from collective.quickupload import logger
from collective.quickupload.browser.quick_upload import get_content_type
from collective.quickupload.browser.quick_upload import getDataFromAllRequests
from collective.quickupload.browser.quick_upload import QuickUploadFile
from collective.quickupload.browser.quick_upload import QuickUploadInit
from collective.quickupload.browser.quick_upload import QuickUploadView
from collective.quickupload.browser.uploadcapable import get_id_from_filename
from collective.quickupload.browser.uploadcapable import INameChooser
from collective.quickupload.browser.uploadcapable import MissingExtension
from collective.quickupload.browser.uploadcapable import QuickUploadCapableFileFactory
from collective.quickupload.browser.uploadcapable import upload_lock
from collective.quickupload.interfaces import IQuickUploadFileFactory
from collective.quickupload.interfaces import IQuickUploadFileSetter
from collective.quickupload.interfaces import IQuickUploadFileUpdater
from imio.annex.quickupload import utils
from plone.i18n.normalizer.interfaces import IUserPreferredFileNameNormalizer
from Products.CMFCore.permissions import ModifyPortalContent
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.utils import base_hasattr
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from z3c.form.validator import Data
from ZODB.POSException import ConflictError
from zope.event import notify
from zope.lifecycleevent import ObjectAddedEvent

import json
import pkg_resources
import urllib


try:
    pkg_resources.get_distribution('plone.uuid')
    from plone.uuid.interfaces import IUUID
    HAS_UUID = True
except pkg_resources.DistributionNotFound:
    HAS_UUID = False


class QuickUploadPortletView(QuickUploadView):
    template = ViewPageTemplateFile("templates/quick_upload.pt")

    @property
    def typeupload(self):
        context = aq_inner(self.context)
        config = context.restrictedTraverse('@@quick_upload_init')
        config.uploader_id = self.uploader_id
        return config.upload_settings().get('typeupload')

    @property
    def is_iconified_categorized(self):
        return utils.is_iconified_categorized(self.typeupload)

    def script_content(self):
        result = super(QuickUploadPortletView, self).script_content()
        return u"""
{0}
jQuery('a#copy_categories').click(PloneQuickUpload.extendCategories);
        """.format(result)


class QuickUploadFileInit(QuickUploadInit):

    def upload_settings(self):
        # if not in @@finder_upload (adding an image in CKeditor for example)
        # make sure we do not have a mediaupload in the SESSION or it is used
        # to determinate media format (image) and it keeps media if adding an image
        # using CKeditor then adding an annex
        if self.request.get('PUBLISHED').__name__ == 'quick_upload':
            session = self.request.get('SESSION', '')
            for session_key in ('mediaupload', 'typeupload'):
                if session_key in session.keys():
                    del session[session_key]
        return super(QuickUploadFileInit, self).upload_settings()


def _check_validateFileIsPDF(obj, request, portal_type, content_type):
    """Check if used content_category requires a PDF file only if adding an
       annex and p_content_type is not already a PDF."""
    if portal_type in ['annex', 'annexDecision'] and content_type != 'application/pdf':
        data = Data([], [], [])
        data.__context__ = obj
        content_category = getDataFromAllRequests(request, 'content_category') or ''
        data.content_category = content_category
        data.contentType = content_type
        if portal_type == 'annexDecision':
            request.set('force_use_item_decision_annexes_group', True)
        validateFileIsPDF(data)
        if portal_type == 'annexDecision':
            request.set('force_use_item_decision_annexes_group', False)


class QuickUploadFileView(QuickUploadFile):

    def _manage_extra_parameters(self, request, f):
        """Manage extra parameters, particularly content_category."""
        # Extra parameters
        content_category = getDataFromAllRequests(request, 'content_category') or ''
        # Add an extra parameter
        if f['success'] and content_category:
            obj = f['success']
            obj.content_category = content_category
            # elements using content_category are initialized in the object created event
            notify(ObjectAddedEvent(obj))

    def quick_upload_file(self):
        """Copied from collective.quickupload"""
        context = aq_inner(self.context)
        request = self.request
        response = request.RESPONSE

        response.setHeader('Expires', 'Sat, 1 Jan 2000 00:00:00 GMT')
        response.setHeader('Cache-control', 'no-cache')
        # application/json is not supported by old IEs but text/html fails in
        # every browser with plone.protect 3.0.11
        response.setHeader('Content-Type', 'application/json; charset=utf-8')
        # disable diazo themes and csrf protection
        request.response.setHeader('X-Theme-Disabled', 'True')

        if request.HTTP_X_REQUESTED_WITH:
            # using ajax upload
            file_name = urllib.unquote(request.HTTP_X_FILE_NAME)
            upload_with = "XHR"
            try:
                file = request.BODYFILE
                file_data = file.read()
                file.seek(0)
            except AttributeError:
                # in case of cancel during xhr upload
                logger.error("Upload of %s has been aborted", file_name)
                # not really useful here since the upload block
                # is removed by "cancel" action, but
                # could be useful if someone change the js behavior
                return json.dumps({u'error': u'emptyError'})
            except:
                logger.error(
                    "Error when trying to read the file %s in request",
                    file_name
                )
                return json.dumps({u'error': u'serverError'})
        else:
            # using classic form post method (MSIE<=8)
            file = request.get("qqfile", None)
            file_data = file.read()
            file.seek(0)
            filename = getattr(file, 'filename', '')
            file_name = filename.split("\\")[-1]
            try:
                file_name = file_name.decode('utf-8')
            except UnicodeDecodeError:
                pass

            file_name = IUserPreferredFileNameNormalizer(
                self.request
            ).normalize(file_name)
            upload_with = "CLASSIC FORM POST"
            # we must test the file size in this case (no client test)
            if not self._check_file_size(file):
                logger.info("Test file size: the file %s is too big, upload "
                            "rejected" % filename)
                return json.dumps({u'error': u'sizeError'})

        # overwrite file
        try:
            newid = get_id_from_filename(
                file_name, context, unique=self.qup_prefs.object_unique_id)
        except MissingExtension:
            return json.dumps({u'error': u'missingExtension'})

        if (newid in context or file_name in context) and \
                not self.qup_prefs.object_unique_id:
            updated_object = context.get(newid, False) or context[file_name]
            mtool = getToolByName(context, 'portal_membership')
            override_setting = self.qup_prefs.object_override
            if override_setting and\
                    mtool.checkPermission(ModifyPortalContent, updated_object):
                can_overwrite = True
            else:
                can_overwrite = False

            if not can_overwrite:
                logger.debug(
                    "The file id for %s already exists, upload rejected"
                    % file_name
                )
                return json.dumps({u'error': u'serverErrorAlreadyExists'})

            overwritten_file = updated_object
        else:
            overwritten_file = None

        content_type = get_content_type(context, file_data, file_name)

        portal_type = getDataFromAllRequests(request, 'typeupload') or ''
        title = getDataFromAllRequests(request, 'title') or ''
        description = getDataFromAllRequests(request, 'description') or ''
        if not title.strip() and self.qup_prefs.id_as_title:
            title = newid

        if not portal_type:
            ctr = getToolByName(context, 'content_type_registry')
            portal_type = ctr.findTypeName(
                file_name.lower(), content_type, ''
            ) or 'File'

        if file_data:
            if overwritten_file is not None:
                updater = IQuickUploadFileUpdater(context)
                logger.info(
                    "reuploading %s file with %s: title=%s, description=%s, "
                    "content_type=%s"
                    % (overwritten_file.absolute_url(), upload_with, title,
                       description, content_type))
                try:
                    self.request.set('defer_categorized_content_created_event', True)
                    # check if PDF before new object is created
                    _check_validateFileIsPDF(self.context, self.request, portal_type, content_type)
                    f = updater(overwritten_file, file_name, title,
                                description, content_type, file_data)
                    self.request.set('defer_categorized_content_created_event', False)
                    # manage extra parameters
                    self._manage_extra_parameters(request, f)
                except ConflictError:
                    # Allow Zope to retry up to three times, and if that still
                    # fails, handle ConflictErrors on client side if necessary
                    raise
                except Exception as e:
                    logger.error(
                        "Error updating %s file: %s", file_name, str(e)
                    )
                    return json.dumps({u'error': u'serverError'})

            else:
                factory = IQuickUploadFileFactory(context)
                logger.info(
                    "uploading file with %s: filename=%s, title=%s, "
                    "description=%s, content_type=%s, portal_type=%s"
                    % (upload_with, file_name, title,
                       description, content_type, portal_type))
                try:
                    self.request.set('defer_categorized_content_created_event', True)
                    # check if PDF before new object is created
                    _check_validateFileIsPDF(self.context, self.request, portal_type, content_type)
                    f = factory(file_name, title, description, content_type,
                                file_data, portal_type)
                    self.request.set('defer_categorized_content_created_event', False)
                    # manage extra parameters
                    self._manage_extra_parameters(request, f)
                except ConflictError:
                    # Allow Zope to retry up to three times, and if that still
                    # fails, handle ConflictErrors on client side if necessary
                    raise
                except Exception as e:
                    logger.error(
                        "Error creating %s file: %s", file_name, str(e)
                    )
                    return json.dumps({u'error': u'serverError'})

            if f['success'] is not None:
                o = f['success']
                logger.info("file url: %s" % o.absolute_url())
                if HAS_UUID:
                    uid = IUUID(o)
                else:
                    uid = o.UID()

                msg = {
                    u'success': True,
                    u'uid': uid,
                    u'name': o.getId(),
                    u'title': o.pretty_title_or_id()
                }
            else:
                msg = {u'error': f['error']}
        else:
            msg = {u'error': u'emptyError'}

        return json.dumps(msg)


class ImioAnnexQuickUploadCapableFileFactory(QuickUploadCapableFileFactory):

    def __call__(self, filename, title, description, content_type, data,
                 portal_type):
        context = aq_inner(self.context)
        error = ''
        result = {}
        result['success'] = None
        newid = get_id_from_filename(filename, context)
        chooser = INameChooser(context)
        newid = chooser.chooseName(newid, context)
        # consolidation because it's different upon Plone versions
        if not title:
            # try to split filenames because we don't want
            # big titles without spaces
            title = filename.rsplit('.', 1)[0]\
                .replace('_', ' ')\
                .replace('-', ' ')

        if newid in context:
            # only here for flashupload method since a check_id is done
            # in standard uploader - see also XXX in quick_upload.py
            raise NameError('Object id %s already exists' % newid)
        else:
            upload_lock.acquire()
            try:
                # XXX begin change by imio.annex
                # this will lead to annex without a content_category because
                # this annex is created by a separated XHR request
                # transaction.begin()
                # XXX end change by imio.annex
                try:
                    context.invokeFactory(type_name=portal_type, id=newid,
                                          title=title, description=description)
                except Unauthorized:
                    error = u'serverErrorNoPermission'
                except ValueError:
                    error = u'serverErrorDisallowedType'
                except Exception as e:
                    error = u'serverError'
                    logger.exception(e)

                if error:
                    if error == u'serverError':
                        logger.info(
                            "An error happens with setId from filename, "
                            "the file has been created with a bad id, "
                            "can't find %s", newid)
                else:
                    obj = getattr(context, newid)
                    if obj:
                        error = IQuickUploadFileSetter(obj).set(
                            data, filename, content_type
                        )
                        # XXX begin change by imio.annex
                        if base_hasattr(obj, 'processForm'):
                            # Archetypes
                            obj._at_rename_after_creation = False
                            obj.processForm()
                            del obj._at_rename_after_creation
                        else:
                            # Dexterity
                            if obj.REQUEST.get('defer_update_categorized_elements', False):
                                notify(ObjectAddedEvent(obj))
                        # XXX end change by imio.annex

                # XXX begin change by imio.annex
                # TODO : rollback if there has been an error
                # transaction.commit()
                # XXX end change by imio.annex
            finally:
                upload_lock.release()

        result['error'] = error
        if not error:
            result['success'] = obj

        return result



# -*- coding: utf-8 -*-

from collective.iconifiedcategory.utils import get_category_icon_url
from collective.iconifiedcategory.utils import get_category_object
from imio.prettylink.adapters import PrettyLinkAdapter
from zope.i18n import translate


class AnnexPrettyLinkAdapter(PrettyLinkAdapter):
    """
    """

    def _get_url(self):
        """ """
        if self.is_preview:
            url = self.context.absolute_url()
            return u"{0}/documentviewer#document/#/p1".format(url)
        else:
            return super(AnnexPrettyLinkAdapter, self)._get_url()

    def _leadingIcons(self):
        """
          Manage icons to display before the annex title.
        """
        res = []
        parent = self.context.aq_parent
        element = parent.categorized_elements[self.context.UID()]
        self.infos = parent.unrestrictedTraverse('@@categorized-childs-infos')

        # preview in progress
        if element["preview_status"] == 'in_progress':
            res.append(
                ('spinner_small.gif',
                 translate("The document is currently under conversion, "
                           "please refresh the page in a few minutes",
                           domain="collective.iconifiedcategory",
                           context=self.request)))
        # category icon
        category = get_category_object(self.context, self.context.content_category)
        category_url = get_category_icon_url(category)
        res.append((category_url,
                    category.title))

        # is a preview, store is_preview as it is reused in _get_url
        self.is_preview = self.infos.show_preview(element)
        if self.is_preview:
            res.append(('file_icon.png',
                        translate(
                            "Preview",
                            domain="collective.iconifiedcategory",
                            context=self.request)))
        return res



# -*- coding: utf-8 -*-

from collective.eeafaceted.z3ctable.columns import ActionsColumn as DashboardActionsColumn
from collective.eeafaceted.z3ctable.columns import BaseColumn
from collective.eeafaceted.z3ctable.columns import DateColumn
from collective.eeafaceted.z3ctable.columns import MemberIdColumn
from collective.eeafaceted.z3ctable.columns import PrettyLinkColumn as DashboardPrettyLinkColumn
from collective.iconifiedcategory.browser.tabview import AuthorColumn as IconifiedAuthorColumn
from collective.iconifiedcategory.browser.tabview import CategoryColumn as IconifiedCategoryColumn
from collective.iconifiedcategory.browser.tabview import CreationDateColumn as IconifiedCreationDateColumn
from collective.iconifiedcategory.browser.tabview import FilesizeColumn as IconifiedFilesizeColumn
from collective.iconifiedcategory.browser.tabview import LastModificationColumn as IconifiedLastModificationColumn
from collective.iconifiedcategory.interfaces import IIconifiedCategorySettings
from imio.annex import _
from plone import api
from Products.CMFPlone.utils import safe_unicode
from zope.i18n import translate

import html


class PrettyLinkColumn(DashboardPrettyLinkColumn):
    header = _(u'Title')
    weight = 20

    # link to a file, open in new tab
    params = {'target': '_blank'}

    def getPrettyLink(self, obj):
        """Display the description just under the pretty link."""
        pl = super(PrettyLinkColumn, self).getPrettyLink(obj)

        # simple blank space
        blank = u'<p class="discreet"></p>'

        # display description if any
        description = safe_unicode(html.escape(obj.Description() or u'')).replace('\n', '<br/>')
        if description:
            description = u'<p class="discreet">{0}</p>'.format(description)

        # display filename if any
        filename = html.escape(obj.file.filename or '')
        field_name = translate(
            'File',
            domain='imio.annex',
            context=obj.REQUEST)
        filename = u'<div class="discreet"><label class="horizontal">{0}</label>' \
            '<div class="type-text-widget">{1}</div></div>'.format(
                field_name, filename)

        # display scan_id if any
        scan_id = html.escape(getattr(obj, "scan_id", '') or '')
        if scan_id:
            field_name = translate(
                'scan_id',
                domain='collective.dms.scanbehavior',
                context=obj.REQUEST)
            scan_id = u'<div class="discreet"><label class="horizontal">{0}</label>' \
                '<div class="type-textarea-widget">{1}</div></div>'.format(
                    field_name, scan_id)
        return pl + blank + description + filename + scan_id


class AuthorColumn(MemberIdColumn):
    """ """
    weight = IconifiedAuthorColumn.weight
    header = IconifiedAuthorColumn.header


class CategoryColumn(IconifiedCategoryColumn, BaseColumn):
    """ """


class CreationDateColumn(IconifiedCreationDateColumn, DateColumn):
    """ """


class LastModificationColumn(IconifiedLastModificationColumn, DateColumn):
    """ """


class FilesizeColumn(IconifiedFilesizeColumn, BaseColumn):
    """ """


class ActionsColumn(DashboardActionsColumn):
    header = _(u'Actions')
    weight = 100
    params = {'showHistory': True, 'showActions': True, 'showArrows': True}

    def _showArrows(self):
        sort_categorized_tab = api.portal.get_registry_record(
            'sort_categorized_tab',
            interface=IIconifiedCategorySettings,
        )
        return not (bool(sort_categorized_tab))

    def renderCell(self, item):
        """ """
        self.params['showArrows'] = self._showArrows()
        return super(ActionsColumn, self).renderCell(item)



# -*- coding: utf-8 -*-

from collective.eeafaceted.batchactions import _ as _CEBA
from collective.eeafaceted.batchactions.browser.views import BaseBatchActionForm
from collective.iconifiedcategory.config import get_sort_categorized_tab
from collective.iconifiedcategory.utils import calculate_filesize
from collective.iconifiedcategory.utils import get_categorized_elements
from imio.annex import _
from imio.annex import logger
from imio.annex.content.annex import IAnnex
from io import BytesIO
from plone import api
from plone.rfc822.interfaces import IPrimaryFieldInfo
from Products.CMFPlone.utils import safe_unicode
from Products.PloneMeeting.widgets.pm_checkbox import PMCheckBoxFieldWidget
from PyPDF2 import PdfFileReader
from PyPDF2 import PdfFileWriter
from PyPDF2.utils import PdfReadError
from z3c.form.browser.radio import RadioFieldWidget
from z3c.form.field import Fields
from zope import schema
from zope.i18n import translate

import zipfile


class DownloadAnnexesBatchActionForm(BaseBatchActionForm):

    label = _CEBA("download-annexes-batch-action-but")
    button_with_icon = True
    apply_button_title = _CEBA('Download')
    section = "annexes"
    # gives a human readable size of "50.0 Mb"
    MAX_TOTAL_SIZE = 52428800

    def _max_total_size(self):
        """ """
        return self.MAX_TOTAL_SIZE

    @property
    def description(self):
        """ """
        descr = super(DownloadAnnexesBatchActionForm, self).description
        descr = translate(descr, domain=descr.domain, context=self.request)
        if self.total_size > self._max_total_size():
            readable_max_size = calculate_filesize(self._max_total_size())
            descr += translate(
                '<p class="warn_filesize">The maximum size you may download at '
                'one time is ${max_size}, here your download size is ${total_size}. '
                'Please unselect some elements, especially large elements for '
                'which size is displayed in red, download it separately.<p>',
                mapping={'max_size': readable_max_size,
                         'total_size': calculate_filesize(self.total_size)},
                domain="collective.eeafaceted.batchactions",
                context=self.request)
        elif self.annex_not_downloadable is not None:
            descr += translate(
                '<p style="color: red;">You selected some elements that are '
                'not downloadable, check for example "${annex_title}". '
                'Please unselect elements that are only previewable.</p>',
                mapping={'annex_title': self.annex_not_downloadable.Title()},
                domain="collective.eeafaceted.batchactions",
                context=self.request)
        else:
            descr += translate(
                '<p>This will download the selected elements as a Zip file.</p>'
                '<p>The total file size is <b>${total_size}</b>, when clicking '
                'on "${button_title}", you will have a spinner, wait until the '
                'Zip file is available.</p>',
                mapping={
                    'total_size': calculate_filesize(self.total_size),
                    'button_title': translate(
                        self.apply_button_title,
                        domain="collective.eeafaceted.batchactions",
                        context=self.request)},
                domain="collective.eeafaceted.batchactions",
                context=self.request)
        return descr

    def _check_annex_not_downloadable(self):
        """ """
        self.annex_not_downloadable = None
        for brain in self.brains:
            obj = brain.getObject()
            if not obj.show_download():
                self.annex_not_downloadable = obj
                self.do_apply = False
                return

    def _check_total_size(self):
        """ """
        self.total_size = self._total_size()
        if self.total_size > self._max_total_size():
            self.do_apply = False

    def _total_size(self):
        """ """
        total = 0
        for brain in self.brains:
            obj = brain.getObject()
            primary_field = IPrimaryFieldInfo(obj)
            size = primary_field.value.size
            total += size
        return total

    def _update(self):
        """Can not apply action if total size exceeded."""
        self._check_total_size()
        if self.do_apply:
            self._check_annex_not_downloadable()

    def available(self):
        """ """
        return True

    def zipfiles(self, content):
        """Return zipped content."""
        fstream = BytesIO()
        zipper = zipfile.ZipFile(fstream, 'w', zipfile.ZIP_DEFLATED)
        for obj in content:
            if not IAnnex.providedBy(obj):
                continue
            primary_field = IPrimaryFieldInfo(obj)
            data = primary_field.value.data
            filename = primary_field.value.filename
            # can not do without a filename...
            if not filename:
                continue
            zipper.writestr(filename, data)
            created = obj.created()
            zipper.NameToInfo[filename].date_time = (
                created.year(),
                created.month(),
                created.day(),
                created.hour(),
                created.minute(),
                int(created.second()))
        zipper.close()
        return fstream.getvalue()

    def _apply(self, **data):
        """ """
        try:
            return self.do_zip()
        except zipfile.LargeZipFile:
            message = "Too much annexes to zip, try selecting fewer annexes..."
            api.portal.show_message(message, self.request, type="error")
            return self.request.response.redirect(self.context.absolute_url())

    def do_zip(self):
        """ Zip all of the content in this location (context)"""
        self.request.response.setHeader('Content-Type', 'application/zip')
        self.request.response.setHeader(
            'Content-disposition', 'attachment;filename=%s.zip'
            % self.context.getId())
        content = [brain.getObject() for brain in self.brains]
        zipped_content = self.zipfiles(content)
        self.request.set('zip_file_content', zipped_content)
        return zipped_content

    def render(self):
        """ """
        if 'zip_file_content' in self.request:
            return self.request['zip_file_content']
        else:
            return super(DownloadAnnexesBatchActionForm, self).render()


class ConcatenateAnnexesBatchActionForm(BaseBatchActionForm):

    label = _CEBA("concatenate-annexes-batch-action-but")
    button_with_icon = True
    apply_button_title = _CEBA('Download')
    # gives a human readable size of "75.0 Mb"
    MAX_TOTAL_SIZE = 78643200

    def _max_total_size(self):
        """ """
        return self.MAX_TOTAL_SIZE

    @property
    def description(self):
        """ """
        descr = super(ConcatenateAnnexesBatchActionForm, self).description
        descr = translate(descr, domain=descr.domain, context=self.request)
        readable_max_size = calculate_filesize(self._max_total_size())
        descr += translate(
            'concatenate_annexes_batch_action_descr',
            mapping={'max_size': readable_max_size, },
            domain="collective.eeafaceted.batchactions",
            context=self.request)
        return descr

    def _annex_types_vocabulary(self):
        """The name of the vocabulary factory to use for annex_types field."""
        return "collective.iconifiedcategory.every_category_uids"

    def _update(self):
        self.fields += Fields(
            schema.List(
                __name__='annex_types',
                title=_(u'Annex types'),
                value_type=schema.Choice(
                    vocabulary=self._annex_types_vocabulary()),
                required=True),
            schema.Bool(__name__='two_sided',
                        title=_(u'Two-sided?'),
                        description=_(u'descr_two_sided'),
                        default=False,
                        required=False),
        )
        self.fields["annex_types"].widgetFactory = PMCheckBoxFieldWidget
        self.fields["two_sided"].widgetFactory = RadioFieldWidget

    def _total_size(self, annexes):
        """ """
        total = 0
        for annex in annexes:
            primary_field = IPrimaryFieldInfo(annex)
            size = primary_field.value.size
            total += size
        return total

    def _error_obj_title(self, obj):
        """ """
        return obj.Title()

    def _apply(self, **data):
        """ """
        annex_type_uids = data['annex_types']
        # get annexes
        annexes = []
        sort_on = 'getObjPositionInParent' if \
            get_sort_categorized_tab() is False else None
        for brain in self.brains:
            obj = brain.getObject()
            filters = {'contentType': 'application/pdf'}
            for annex_type_uid in annex_type_uids:
                filters['category_uid'] = annex_type_uid
                annexes += get_categorized_elements(
                    obj,
                    result_type='objects',
                    sort_on=sort_on,
                    filters=filters)
        # return if nothing to produce
        if not annexes:
            api.portal.show_message(
                _("Nothing to export."),
                request=self.request)
            return
        # can not generate if total size too large
        total_size = self._total_size(annexes)
        if self._total_size(annexes) > self._max_total_size():
            api.portal.show_message(
                _("concatenate_annexes_pdf_too_large_error",
                  mapping={'total_size': calculate_filesize(total_size),
                           'max_total_size': calculate_filesize(
                            self._max_total_size())}),
                request=self.request,
                type="error")
            return
        # create unique PDF file
        output_writer = PdfFileWriter()
        for annex in annexes:
            try:
                output_writer.appendPagesFromReader(
                    PdfFileReader(BytesIO(annex.file.data), strict=False))
            except PdfReadError as exc:
                api.portal.show_message(
                    _("concatenate_annexes_pdf_read_error",
                      mapping={'annex_title': safe_unicode(annex.Title()),
                               'obj_title': safe_unicode(
                          self._error_obj_title(annex.aq_inner.aq_parent))}),
                    request=self.request,
                    type="error")
                logger.exception(exc)
                self.request.set(
                    'concatenate_annexes_item_pdf_error_url', obj.absolute_url())
                return
            if data['two_sided'] and \
               output_writer.getNumPages() % 2 != 0 and \
               annex != annexes[-1]:
                output_writer.addBlankPage()
        pdf_file_content = BytesIO()
        output_writer.write(pdf_file_content)
        self.request.set('pdf_file_content', pdf_file_content)
        return pdf_file_content

    def render(self):
        if 'pdf_file_content' in self.request:
            filename = "%s-annexes.pdf" % self.context.getId()
            self.request.response.setHeader('Content-Type', 'application/pdf')
            self.request.response.setHeader(
                'Content-disposition', 'attachment; filename=%s' % filename)
            pdf_file_content = self.request['pdf_file_content']
            pdf_file_content.seek(0)
            return pdf_file_content.read()
        elif self.request.RESPONSE.status == 204:
            # return something so the faceted is refrehsed
            return self.request.get('concatenate_annexes_pdf_error_url')
        return super(ConcatenateAnnexesBatchActionForm, self).render()



# -*- coding: utf-8 -*-
"""
imio.annex
----------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from imio.annex import _
from plone.app.contenttypes.content import File
from plone.app.contenttypes.interfaces import IFile
from plone.app.dexterity import PloneMessageFactory as _PMF
from plone.autoform import directives as form
from plone.dexterity.schema import DexteritySchemaPolicy
from plone.namedfile.field import NamedBlobFile
from plone.supermodel import model
from plone.uuid.interfaces import ATTRIBUTE_NAME
from z3c.form.interfaces import IAddForm
from z3c.form.interfaces import IEditForm
from zope import schema
from zope.interface import implements


class IAnnex(model.Schema, IFile):
    """Schema for Annex content type"""

    title = schema.TextLine(
        title=_PMF(u'label_title', default=u'Title'),
        required=False,
    )

    description = schema.Text(
        title=_PMF(u'label_description', default=u'Summary'),
        description=_PMF(
            u'help_description',
            default=u'Used in item listings and search results.'
        ),
        required=False,
        missing_value=u'',
    )

    form.order_before(description='*')
    form.order_before(title='*')

    form.omitted('title', 'description')
    form.no_omit(IEditForm, 'title', 'description')
    form.no_omit(IAddForm, 'title', 'description')

    model.primary('file')
    file = NamedBlobFile(
        title=_(u'File'),
        required=True,
    )


class Annex(File):
    """Annex content type"""
    implements(IAnnex)

    def UID(self):
        """Speedup getting UID."""
        return getattr(self, ATTRIBUTE_NAME)

    def show_download(self):
        """Condition to show the "Download" action."""
        res = True
        parent = self.aq_parent
        element = parent.categorized_elements[self.UID()]
        # preview with protected download
        if element['show_preview'] == 2:
            infos = parent.unrestrictedTraverse('@@categorized-childs-infos')
            res = infos.show_download(element)
        return res

    def show_preview(self):
        """Condition to show the "Preview" action."""
        return self.aq_parent.categorized_elements[self.UID()]['preview_status'] == 'converted'


class AnnexSchemaPolicy(DexteritySchemaPolicy):
    """Schema Policy for Annex"""

    def bases(self, schema_name, tree):
        return (IAnnex, )



# -*- coding: utf-8 -*-

from collective.dms.scanbehavior.behaviors.behaviors import IScanFields
from plone.autoform import directives as form
from plone.autoform.interfaces import IFormFieldProvider
from zope.interface import alsoProvides


class IScanFieldsHiddenToSignAndSigned(IScanFields):

    form.omitted('to_sign')
    form.omitted('signed')

alsoProvides(IScanFieldsHiddenToSignAndSigned, IFormFieldProvider)



# -*- coding: utf-8 -*-
"""
imio.annex
----------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from zope.interface import Interface
from zope.publisher.interfaces.browser import IDefaultBrowserLayer


class IImioAnnexLayer(IDefaultBrowserLayer):
    """Marker interface that defines a browser layer."""


class IAnnexFileChangedEvent(Interface):
    pass


class IConversionStartedEvent(Interface):
    pass


class IConversionReallyFinishedEvent(Interface):
    pass



# -*- coding: utf-8 -*-
"""
imio.annex
----------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from collective.documentviewer.async import JobRunner
from collective.documentviewer.convert import Converter
from imio.annex.events import ConversionReallyFinishedEvent
from imio.annex.events import ConversionStartedEvent
from zope.event import notify


def converter_call(self, *args, **kwargs):
    notify(ConversionStartedEvent(self.context))
    res = Converter._old___call__(self, *args, **kwargs)
    # in collective.documentviewer, ConversionFinishedEvent is called before
    # information "converting" is set back to "False", we need to have it
    # to update the preview_status
    notify(ConversionReallyFinishedEvent(self.context))
    return res


def jobrunner_queue_it(self):
    JobRunner._old_queue_it(self)
    notify(ConversionStartedEvent(self.object))
